#ifndef INC_ADBMS_ABSTRACTION_H_
#define INC_ADBMS_ABSTRACTION_H_

#include "ADBMS_Driver.h"
#include "ADBMS_Intern.h"

#define mV_from_ADBMS6830(x) (((((int16_t)(x))) * 0.150) + 1500)

// Macro to convert voltage in mV to the internal format for under/over voltage thresholds.
// Calculation: internal = (mV - 1500) / (16 * 0.15) = (mV - 1500) / 2.4.
// To perform integer arithmetic with rounding, we compute:
//   internal = ((mV - 1500) * 5 + 6) / 12
#define mV_to_ADBMS6830(mV) ((((mV) - 1500) * 5 + 6) / 12)

ADBMS_Internal_Status amsReset();

ADBMS_Internal_Status initAMS(USER_PTR_TYPE ptr);
ADBMS_Internal_Status amsWakeUp();

ADBMS_Internal_Status amsCellMeasurement(BMS_Chip (*module)[N_BMS]);

ADBMS_Internal_Status amsAuxAndStatusMeasurement(BMS_Chip (*module)[N_BMS]);

ADBMS_Internal_Status amsConfigBalancing(const uint32_t channels[static N_BMS], uint8_t dutyCycle);
ADBMS_Internal_Status amsStartBalancing();
ADBMS_Internal_Status amsStopBalancing();

ADBMS_Internal_Status amsSelfTest();

ADBMS_Internal_Status amsConfigOverUnderVoltage(uint16_t overVoltage, uint16_t underVoltage); //arguments in mV

ADBMS_Internal_Status amsCheckUnderOverVoltage(BMS_Chip (*module)[N_BMS]);

ADBMS_Internal_Status amsClearFlag();
ADBMS_Internal_Status amsClearAux();
ADBMS_Internal_Status amsClearOVUV();

ADBMS_Internal_Status amsReadCellVoltages(BMS_Chip (*module)[N_BMS]);

#endif /* INC_ADBMS_ABSTRACTION_H_ */
