#pragma once
#ifndef __ADBMS_INTERN_H
#define __ADBMS_INTERN_H
#include <stdint.h>

#include "stm32h7xx_hal.h"
#include "main.h"

typedef enum {
    ADBMS_OK = 0,
    ADBMS_ERROR,
    ADBMS_BUSY,
    ADBMS_TIMEOUT,
} ADBMS_Internal_Status;

[[maybe_unused, gnu::always_inline]]
static inline void mcuAdbmsCSLow() {
    HAL_GPIO_WritePin(AMS_CS_GPIO_Port, AMS_CS_Pin, GPIO_PIN_RESET);
}

[[maybe_unused, gnu::always_inline]]
static inline void mcuAdbmsCSHigh() {
    HAL_GPIO_WritePin(AMS_CS_GPIO_Port, AMS_CS_Pin, GPIO_PIN_SET);
}

// user_ptr is passed to AMS_Init and could be used to pass the SPI handle
// the BMS code does not modify it in any way
#define USER_PTR_TYPE SPI_HandleTypeDef*

[[maybe_unused, gnu::always_inline, gnu::access(read_only, 2, 3), gnu::nonnull(1)]]
static inline ADBMS_Internal_Status mcuSPITransmit(USER_PTR_TYPE user_ptr, const uint8_t* buffer, uint8_t buffersize, uint32_t timeout) {
    const HAL_StatusTypeDef status = HAL_SPI_Transmit(user_ptr, buffer, buffersize, timeout);
    return (status == HAL_OK) ? ADBMS_OK : ADBMS_ERROR;
}

[[maybe_unused, gnu::always_inline, gnu::access(read_write, 2, 3), gnu::nonnull(1)]]
static inline ADBMS_Internal_Status mcuSPIReceive(USER_PTR_TYPE user_ptr, uint8_t* buffer, uint8_t buffersize, uint32_t timeout) {
    const HAL_StatusTypeDef status = HAL_SPI_Receive(user_ptr, buffer, buffersize, timeout);
    return (status == HAL_OK) ? ADBMS_OK : ADBMS_ERROR;
}

[[maybe_unused, gnu::always_inline, gnu::access(read_write, 2, 4), gnu::access(read_only, 3, 4), gnu::nonnull(1, 2)]]
static inline ADBMS_Internal_Status mcuSPITransmitReceive(USER_PTR_TYPE user_ptr, uint8_t* rxbuffer, const uint8_t* txbuffer, uint8_t buffersize, uint32_t timeout) {
    const HAL_StatusTypeDef status = HAL_SPI_TransmitReceive(user_ptr, txbuffer, rxbuffer, buffersize, timeout);
    return (status == HAL_OK) ? ADBMS_OK : ADBMS_ERROR;
}

// Delay by `delay` milliseconds
[[maybe_unused, gnu::always_inline]]
static inline void mcuDelay(uint32_t delay) {
    HAL_Delay(delay);
}

// Optional logging, leave as is if not needed

#ifndef ADBMS_NO_LOGGING_DEFS

#define LOG_PREFIX "[ADBMS] "
#include "log.h"

enum {
    ADBMS_LOG_LEVEL_FATAL = 0,
    ADBMS_LOG_LEVEL_ERROR,
    ADBMS_LOG_LEVEL_WARNING,
    ADBMS_LOG_LEVEL_INFO,
    ADBMS_LOG_LEVEL_DEBUG,
    ADBMS_LOG_LEVEL_NOISY,
};

// Returns a timestamp in milliseconds
// Only used for logging
[[maybe_unused, gnu::always_inline]]
static inline uint32_t mcuGetTime() {
    return HAL_GetTick();
}

#define debug_log(level, ...) log_message((log_level_t)level, __VA_ARGS__)
#define debug_log_cont(level, ...) log_message_cont((log_level_t)level, __VA_ARGS__)
#define DEBUG_CHANNEL_ENABLED(level) log_is_level_enabled((log_level_t)level)

#endif // ADBMS_NO_LOGGING_DEFS

#endif // __ADBMS_INTERN_H